'''
Created on Nov 3, 2012

@author: Gokcen
'''

from ApicoAMP_helper import *
import os, sys
from optparse import OptionParser

NON_ATMP = 1
ATMP = 2
NO_TM = 3
UNKNOWN = 4
NO_GO = 5
#indexes they sit on are associated with the above constant values
PREDICTION_CODES = ["", "non-ApicoTMP", "ApicoTMP", "noTM", "unknownId", "noGO"] #["", "0", "1", "-1", "-2"] 
FEATURE_COUNT = 59

TM_STATUS_UNKNOWN = 1
TM_STATUS_NO_TM = 2
TM_STATUS_TM = 3

DEBUG = 1 #1
possibleVoteThresholds = [6, 7, 8, 9, 10]

class goTermMapInstance:
    def __init__(self, givenId, posProb, negProb):
        self.givenId = givenId
        self.posProb = posProb
        self.negProb = negProb
 
class TM_portions:
    def __init__(self, n,c,core):
        self.n_list = n
        self.c_list = c
        self.core_list = core
    
class TMFeatureExtract:
    
    def __init__(self, gesFile):
        self.aaList = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']
        self.aaListMap = {}
        for i in range(len(self.aaList)):
            self.aaListMap[self.aaList[i]] = i
        self.GES = readGESindex(gesFile)

    def _aaFreqFeatureMap_single(self, TM):
        '''Calculates frequencies of each aa in the given seq'''
        featureVector = []
        for i in range(len(self.aaList)):
            featureVector.append(0.0);
            
        for c in TM:
            featureVector[self.aaListMap[c]] += 1.0
        for i in range(len(featureVector)):
            if len(TM):
                featureVector[i] /= float(len(TM))
        return featureVector

    def _aaFreqFeatureMap_batch(self, nonEmptyTMs):
        '''
        Each seq contains one or more TM region separated by '/'
        Extract naive features(freq of each aa) from each TM region
        And, outputs the element-wise averages of these feature vectors 
        over each TM region
        '''
        featureVector = []
        for i in range(len(self.aaList)):
            featureVector.append(0.0);
        
        for TM in nonEmptyTMs: 
            fv = self._aaFreqFeatureMap_single(TM)
            #accumulate fv on featureVector
            for i in range(len(featureVector)):
                featureVector[i] += fv[i]
        #take the avg            
        for i in range(len(featureVector)):
            if len(nonEmptyTMs):
                featureVector[i] /= len(nonEmptyTMs);           
        return featureVector

    def _chunkyFeatureExtract(self, nonEmptyTMs):
        #extract freq features
        featureVectorG = self._aaFreqFeatureMap_batch(nonEmptyTMs)
        #add length feature
        featureVectorG.append(self._lengthParameterCalculate(nonEmptyTMs))
        #add
        chunkCounts = [1,2,3,4,5,6,7,8]
        for chunkN in chunkCounts:
            for i in range(chunkN):
                featureVectorG.append(self._avgGESCalculate(nonEmptyTMs, chunkN, i))
        return featureVectorG

    def _lengthParameterCalculate(self, nonEmptyTMs):
        '''
        Calculates avg length of TM regions
        '''        
        avgLength = 0.0
        for TM in nonEmptyTMs: 
            avgLength += len(TM)
        return avgLength/float(len(nonEmptyTMs))

    def _avgGESCalculate(self, nonEmptyTMs, portionCount, portionIndex):
        '''Avg GES scores over portions of seq(portionIndex'th portion of 
        seq that is divided into 'portionCount' portions'''
        avgGes = 0.0
        for seq in nonEmptyTMs:
            
            #select described portion of seq
            portionLength = len(seq)/portionCount
            startIndex = portionIndex * portionLength
            endIndex = (portionIndex + 1) * portionLength
            if (portionIndex == portionCount-1):
                endIndex = len(seq)
            seq = seq[startIndex : endIndex]
            if len(seq) != 0:
                avgOverTM = 0.0
                for s in seq:
                    avgOverTM += self.GES[s]
                avgOverTM /= len(seq) #avg over a TM region
                avgGes += avgOverTM
        if len(nonEmptyTMs):
            avgGes /= float(len(nonEmptyTMs)) #avg over all TM regions given
        '''
        Partial belonging to a chunk was tolerated below, above approach works better
        for seq in nonEmptyTMs:
            portionLength = len(seq)/float(portionCount)
            startIndex = portionIndex * portionLength
            startIndex_floor = int(math.floor(startIndex))
            avgOverTM = self.GES[seq[startIndex_floor]] * (startIndex - startIndex_floor)
            moreToGo = portionLength - (startIndex - startIndex_floor)
            moreToGo_floor = int(math.floor(startIndex))
            unTouchedIndex = startIndex_floor + 1
            for i in range(moreToGo_floor):
                index = startIndex_floor+1+i
                if index < len(seq):
                    avgOverTM += self.GES[seq[index]]
                    unTouchedIndex += 1
            if unTouchedIndex < len(seq):
                avgOverTM += self.GES[seq[unTouchedIndex]] * (moreToGo - moreToGo_floor)   
            avgOverTM /= portionLength #avg over a TM region
            avgGes += avgOverTM
        avgGes /= float(len(nonEmptyTMs)) #avg over all TM regions given'''
        return avgGes
        
    def mapSeqsToFeatureSpace(self, TM_portions):
        '''
        Main function called from outside!
        '''
        if TM_portions.core_list == []:
            fv = [0]*FEATURE_COUNT
            #print 'oyy'
            return fv
        fv = [] 
        fv.append(self._chunkyFeatureExtract(TM_portions.core_list))
        for TM_list in [TM_portions.n_list, TM_portions.c_list]:
            fv.append([self._avgGESCalculate(TM_list, 1, 0)])
        
        return mergeVectorRepr(fv)
 
class topologySeq:
    def __init__(self, id, topology, sequence):
        self.id = id
        self.sequence = sequence
        self.topology = topology
            
class TM_info:
    def __init__(self, startIndex, length, isCytoplasmic):
        self.startIndex = startIndex
        self.length = length
        self.isCytoplasmic = isCytoplasmic
        
    def print_simple(self):
        print("{} {} {}".format(self.startIndex, self.length, self.isCytoplasmic))

class TMtopologyHandle:
    def __init__(self, idTMtopologyFile, indentForCoreAnalysis, prePostRegionL, gesFilePath):
        self.GES = readGESindex(gesFilePath)
        self.idTopologyMap = self._readIdTMTopologyFile(idTMtopologyFile)
        self.indentForCoreAnalysis = indentForCoreAnalysis
        self.prePostRegionL = prePostRegionL
        
    def parseTMTopology(self, topology):
        if topology == "-":
            return []
        SP = 0
        if len(topology.split('/')) > 1:
            topology = topology.split('/')[1]
            SP = 1   
        start = 1
        TM_list = []
        myBuffer = ""
        iORo = ""
        TM_s = 0
        TM_e = 0
        for ch in topology:
            #print(ch, SP, start)    
            if ch.isdigit() and SP: #inside SP cleav info
                pass
            elif ch in ['i', 'o']:
                if not start:  # record the previous TM
                    TM_e = int(myBuffer)-1 #adjust index, now starts from zero
                    if iORo == 'i':
                        isCytoplasmic = 1
                    elif iORo == 'o':
                        isCytoplasmic = 0
                    TM_list.append(TM_info(TM_s, TM_e - TM_s + 1, isCytoplasmic))
                else:
                    start = 0
                if SP:
                    SP = 0
                iORo = ch
                myBuffer = ""
            elif ch.isdigit():
                myBuffer += ch
            elif ch == '-':
                TM_s = int(myBuffer)-1 #adjust index, now starts from zero
                myBuffer = ""
        '''for el in TM_list:
            el.print_simple()'''
        return TM_list
    
    def findCore(self, seq, GES):
        windowSize = 5
        halfWindowSize = windowSize/2
        mappedSeq = []
        for i in range(halfWindowSize, len(seq) - windowSize):
            avgGES = 0.0
            for j in range(i, i+windowSize):
                if seq[j] in GES:
                    avgGES += GES[seq[j]]
            mappedSeq.append(avgGES/windowSize)
    
        start = -1
        '''for i in range(halfWindowSize):
            if GES[seq[i]] > 8:
                start = i
                break'''
        if start == -1:
            for i in range(len(mappedSeq)):
                if mappedSeq[i] > -0.94 or ( (seq[i + halfWindowSize] in GES) and (GES[seq[i + halfWindowSize]] > 8) ):
                    start = i + halfWindowSize
                    break
    
        if start == -1:
            print("Start of hyd. core cannot be found")
            return -1, -1
    
        end = -1
        for i in range(len(mappedSeq)-1, -1, -1):
            if mappedSeq[i] > -0.94 or ( (seq[i - halfWindowSize] in GES) and (GES[seq[i - halfWindowSize]] > 8) ):
                end = i - halfWindowSize
                break
    
        if end == -1:
            print("End of hyd. core cannot be found")
        
        return start, end + 1 #+1 since this index won't be included when seq is cropped
    
    def processInstances(self, id, sequence):
        '''
        no signalp crosscheck,
        +-5 from TMHMM prediction ends of TMs,
        no check of initial parts of -5+5 againsat >8 on halfWindowSize
        '''   

        if not id in self.idTopologyMap:
            #print 'not found id', id
            return TM_STATUS_UNKNOWN, TM_portions([], [], [])  #TODO: Differentiate between cases of no-TM, unknown sequence
        
        if self.idTopologyMap[id].strip() == 'o' or self.idTopologyMap[id].strip() == 'i':
            #print 'nonTM', id
            return TM_STATUS_NO_TM, TM_portions([], [], []) 
          
        instance = topologySeq(id, self.idTopologyMap[id], sequence)
        TM_list = self.parseTMTopology(instance.topology)
           
        n_list = []
        c_list = []
        core_list = []
        
        for i in range(len(TM_list)):
            TM = TM_list[i]        
            '''if int(instance.signalp) >= TM.startIndex: #if TM overlaps with SP, ignore that TM
                lastTMend = int(instance.signalp) + 1
            else:'''
            newStart = TM.startIndex - self.indentForCoreAnalysis
            if newStart < 0: newStart = 0
            newEnd = TM.startIndex + TM.length + self.indentForCoreAnalysis
            if newEnd > len(instance.sequence): newEnd = len(instance.sequence)
            TMseq = instance.sequence[newStart:newEnd]
            if TM.isCytoplasmic != 1: # if not isCytoplasmic, reverse
                TMseq = TMseq[::-1]            
            coreStart, coreEnd = self.findCore(TMseq, self.GES)
            core_list.append(TMseq[coreStart:coreEnd]) 
            
            if TM.isCytoplasmic == 1:
                newStart += coreStart
                newEnd = newStart + (coreEnd - coreStart)
            else:
                newEnd -= coreStart
                newStart = newEnd - (coreEnd - coreStart)

            preRegionStart = newStart - self.prePostRegionL
            if preRegionStart < 0: preRegionStart = 0
            preRegionEnd = newEnd + self.prePostRegionL
            if  preRegionEnd > len(instance.sequence): 
                preRegionEnd = len(instance.sequence) 
            
            n_list.append(instance.sequence[preRegionStart:newStart])
            c_list.append(instance.sequence[newEnd:preRegionEnd])
       
        return TM_STATUS_TM, TM_portions(n_list, c_list, core_list)

    def _readIdTMTopologyFile(self, idTMtopologyFile):
        idTopologyMap = {}
        infile = open(idTMtopologyFile, 'r')
        for line in infile:
            line = line.strip()
            if line:
                record = line.split('\t')
                if len(record) == 2:
                    idTopologyMap[record[0].strip()] = record[1].strip()
                else:
                    print line
        infile.close()
        return idTopologyMap 
 
class ApicoAMP_GO_classifier:
    
    def __init__(self, goTermFeatureMapFile, proteinGoTermMapFile):
        self.termMap = self._readGoTermFeatureMap(goTermFeatureMapFile)
        self.proteinGoTermMap = self._readGoTermDatabase(proteinGoTermMapFile)
        
    def _readGoTermFeatureMap(self, mapFile):
        '''
        Parses goTermFeatureMap file that contains entries as follows:
        <GoTerm description><GoTerm id><posProb><negProb>
        Note: posProb and negProb are calculated using whole positive and negative training sets
        This file contains cashed prob info that is gonna be used by Go classifier of ApicoAMP
        Output termMap associates GoTerm descriptions with prob values
        '''
        mapFile = open(mapFile, 'r')
        termMap = {}
        for line in mapFile:
            record = line.split("\t")
            termMap[record[0].strip()] = goTermMapInstance(int(record[1].strip()), \
                                                           float(record[2].strip()), \
                                                           float(record[3].strip()) )
        mapFile.close()
        return termMap
    
    def _readGoTermDatabase(self, proteinGoTermMapFile):
        proteinGoTermMap = {}
        proteinGoTermMapFile = open(proteinGoTermMapFile, 'r')
        for line in proteinGoTermMapFile:
            line = line.strip()
            if line:
                record = line.split('\t')
                proteinId = record[0].strip()
                GoTermList = []
                record = record[1:]
                for term in record:
                    GoTermList.append(term.strip())
                    
                if not proteinId in proteinGoTermMap:
                    proteinGoTermMap[proteinId] = GoTermList
                #else:
                #    print "Opps" + proteinId
        proteinGoTermMapFile.close()
        return proteinGoTermMap
            
    def classify(self, proteinId):
        '''
        Given a list of GoTerms associated with a protein,
        this function returns the classification result
        '''
        if proteinId in self.proteinGoTermMap:
            GOTerms = self.proteinGoTermMap[proteinId]
        else:
            return UNKNOWN, []
        
        #relevantGoTerms = GOTerms
        relevantGoTerms = []
        scores = posNegPair() 
        for term in GOTerms:
            if (term in self.termMap):
                scores.pos += self.termMap[term].posProb
                scores.neg += self.termMap[term].negProb
                relevantGoTerms.append(term)
        
        if scores.pos > scores.neg:
            return ATMP, relevantGoTerms
        elif scores.pos == scores.neg:
            return NO_GO, relevantGoTerms
        else:
            return NON_ATMP, relevantGoTerms
        
    def classify_batch(self, proteinIds):
        predList = []
        relevantGoTerms = []
        for id in proteinIds:
            pred, GOterms = self.classify(id)
            predList.append(pred)
            relevantGoTerms.append(GOterms)
        return predList, relevantGoTerms    
       
class ApicoAMP_TM_classifier_parameters:
    def __init__(self, gesFilePath, idTMtopologyFile, indentForCoreAnalysis, prePostRegionL,
                 whereIsSVM, tempFolderPath, ec, c, g, voteThreshold, svmModelPath,
                 goTermFeatureMapFile, proteinGoTermMapFile):
        self.gesFilePath = gesFilePath
        self.idTMtopologyFile = idTMtopologyFile
        self.indentForCoreAnalysis = indentForCoreAnalysis
        self.prePostRegionL = prePostRegionL
        self.whereIsSVM = whereIsSVM
        self.tempFolderPath  =  tempFolderPath
        self.ec = ec
        self.c = c
        self.g = g
        self.voteThreshold = voteThreshold
        self.svmModelPath = svmModelPath
        self.goTermFeatureMapFile = goTermFeatureMapFile
        self.proteinGoTermMapFile = proteinGoTermMapFile
        
        
class ApicoAMP_TM_classifier:
    
    def __init__(self, parameters):  
        self.parameters = parameters
        self.TMFeatureExtractHandle = TMFeatureExtract(self.parameters.gesFilePath)
        self.TMtopologyHandle = TMtopologyHandle(self.parameters.idTMtopologyFile, 
                                                 self.parameters.indentForCoreAnalysis, 
                                                 self.parameters.prePostRegionL,
                                                 self.parameters.gesFilePath)
        self.svmClassifierHandle = svmClassifier(parameters.whereIsSVM, parameters.tempFolderPath) 
    
    def classify(self, proteinId, proteinSequence):
        TM_status, TM_portions = self._getTMtopology(proteinId, proteinSequence)
        
        if TM_status == TM_STATUS_UNKNOWN:
            return -2
        elif TM_status == TM_STATUS_NO_TM:
            return -1
        fv = self._featureExtract(TM_portions)
        return self.svmClassify([fv])
            
    def svmClassify(self, fvs):
        predictions = []
        for i in range (self.parameters.ec):
            modelFileName = self.parameters.svmModelPath + "model" + str(i)
            normDataFile = self.parameters.svmModelPath + "normDataFile" + str(i) 
            predictions.append(self.svmClassifierHandle.runTest(self.parameters, fvs, normDataFile, modelFileName))      
        return predictions
    
    def classify_batch(self, ids, seqs):
        isTMMap = []
        fv_list = []
        for i in range(len(ids)):
            TM_status, TM_portions = self._getTMtopology(ids[i], seqs[i])
            
            if TM_status == TM_STATUS_UNKNOWN:
                isTMMap.append(0)
            elif TM_status == TM_STATUS_NO_TM:
                isTMMap.append(1)
            else:
                isTMMap.append(2)
        
            fv = self._featureExtract(TM_portions)
            fv_list.append(fv)
        
        preds = self.svmClassify(fv_list)
        for i in range(len(isTMMap)):
            if isTMMap[i] == 0:
                preds[0][i] = -2 #Signal, only 0th ensemble pred is set to -2
            elif isTMMap[i] == 1:
                preds[0][i] = -1 #Signal, only 0th ensemble pred is set to -1
        return preds
        
    def _getTMtopology(self, proteinId, proteinSequence):
        return self.TMtopologyHandle.processInstances(proteinId, proteinSequence)

    def _featureExtract(self, TM_portions):
        fv = self.TMFeatureExtractHandle.mapSeqsToFeatureSpace(TM_portions)
        return fv
    
    def batchFeatureVectorMap(self, ids, seqs):
        fv_list = []
        for i in range(len(ids)):
            TM_portions = self._getTMtopology(ids[i], seqs[i])
            fv = self._featureExtract(TM_portions)
            fv_list.append(fv)
        return fv_list;

class predictionStatus:
    
    def __init__(self, TMstatus, TM_classifierOut, GO_classifierOut, TM_classifierDecisions, GOterms):
        self.TMstatus = TMstatus                     #UNKNOWN, NO_TM, ATMP or NON_ATMP
        self.TM_classifierOut = TM_classifierOut
        self.GO_classifierOut = GO_classifierOut
        self.TM_classifierDecisions = TM_classifierDecisions             #to be used in debug
        self.GOterms = GOterms

def ApicoAMP_classify_helper(parameters, seqs, ids):   
    ApicoAMP_GO_classifierInstance = ApicoAMP_GO_classifier(parameters.goTermFeatureMapFile, parameters.proteinGoTermMapFile) 
    ApicoAMP_TM_classifierInstance = ApicoAMP_TM_classifier(parameters)
    
    predTM = ApicoAMP_TM_classifierInstance.classify_batch(ids, seqs)
    predGO, GOTerms = ApicoAMP_GO_classifierInstance.classify_batch(ids)
    
    idPredictionBundle = {}
 
    for i in range (len(ids)):
        if predTM[0][i] == -2:
            idPredictionBundle[str(ids[i])] = predictionStatus(UNKNOWN, 0, predGO[i], 0, GOTerms[i])
        elif predTM[0][i] == -1:
            idPredictionBundle[str(ids[i])] = predictionStatus(NO_TM, 0, predGO[i], 0, GOTerms[i])
        else:
            voteCount = 0
            for ens in range(parameters.ec):
                if predTM[ens][i] == 1:
                    voteCount += 1
            TM_classifierDecisions = {}
            for voteTh in possibleVoteThresholds:
                if voteCount >= voteTh:
                    TM_classifierDecisions[voteTh] = ATMP
                else:
                    TM_classifierDecisions[voteTh] = NON_ATMP
            
            
            if TM_classifierDecisions[parameters.voteThreshold] == ATMP or predGO[i] == ATMP:
                TMstatus = ATMP
            else:
                TMstatus = NON_ATMP
            
            idPredictionBundle[str(ids[i])] = predictionStatus(TMstatus, voteCount, predGO[i], TM_classifierDecisions,GOTerms[i])
                 
    outStrReturn = ""
    for i in range (len(ids)):
        outStr = str(ids[i]) + "\t" + PREDICTION_CODES[idPredictionBundle[str(ids[i])].TMstatus]  + "\t"
        outStr += str(idPredictionBundle[str(ids[i])].TM_classifierOut)  + "\t"
        outStr += str(PREDICTION_CODES [ idPredictionBundle[str(ids[i])].GO_classifierOut ] )  + "\t"
        if DEBUG:
            outStr += "/".join(idPredictionBundle[str(ids[i])].GOterms)
        outStrReturn += outStr + '\n'
    
    return idPredictionBundle, outStrReturn
 
def ApicoAMP_classify(scriptPath, outDataPath, SVM_binariesPath, seqs, ids, voteThreshold):  
    parameters = ApicoAMP_TM_classifier_parameters(outDataPath + "GESindex", 
                                                   outDataPath + "allApicomplexan_uniqueStrand_11_28_12.idTMtopology_signalpResolved", 
                                                   indentForCoreAnalysis = 5, 
                                                   prePostRegionL = 2,
                                                   whereIsSVM = SVM_binariesPath,  
                                                   tempFolderPath = os.path.join(scriptPath + "svm_trash", ""), 
                                                   ec = 10,
                                                   c = 4, 
                                                   g = 1,
                                                   voteThreshold =  voteThreshold , 
                                                   svmModelPath = os.path.join(outDataPath + "svmModels", ""), 
                                                   goTermFeatureMapFile = outDataPath + "goTermFeatureMap_11_3_12", 
                                                   proteinGoTermMapFile = outDataPath + "AllApicomplexan_uniqueStrand_11_28_12_forRealAll.goTermList")  
    idPredictionBundle, outStrReturn = ApicoAMP_classify_helper(parameters, seqs, ids)
    return idPredictionBundle, outStrReturn
                    
   
def requiredFileCheck(requiredFiles):
    '''
    Checks if required files exist.
    '''
    requiredFileErrorFlag = False
    for item in requiredFiles:
        if not os.path.exists(item):
            requiredFileErrorFlag = True
            print(item)
            break
    return requiredFileErrorFlag
 
def promptUser():
    
    usage = "usage: %prog -f FASTA_FILE -o OUT_FILE [-v VOTE_THRESHOLD]"
    parser = OptionParser(usage=usage)
    parser.add_option("-f", "--fastaFile",
                      action="store", dest="fastaFileName", type="string", metavar="FILE", default="",
                      help="Read FASTA formatted sequences from FILE.")
    parser.add_option("-o", "--outputFile", dest="outFileName", default="",
                      metavar="FILE", help="Write output to FILE.")
    parser.add_option("-v", "--voteThreshold",
                      action="store", dest="voteThreshold", type="string", default="", metavar="THRESHOLD",
                      help="THRESHOLD number of ensembles (out of 10) need to vote positive for a sequence to be predicted as positive" + ". [default: 10]" )
    
    (options, args) = parser.parse_args()
    
    threshold = 10;
    if options.voteThreshold:
        threshold = int(options.voteThreshold)

    if options.fastaFileName == "" or options.outFileName == "" or not os.path.exists(options.fastaFileName) or threshold > 10 or threshold < 0:
        parser.print_help()
        return False
    
    return [options.fastaFileName, options.outFileName, threshold]
 
if __name__ == "__main__":

    #Python version check
    req_version = (2,7)
    cur_version = sys.version_info
    if cur_version.major != req_version[0] or cur_version.minor != req_version[1] :
        print "Python 2.7 is required for ApicoAMP to work. You are using Python " + str(cur_version.major) +  "." + str(cur_version.minor) + ". " + \
        "Please make sure you're using the correct Python version."
    else:       
        print
        
        scriptPath, scriptname = os.path.split(sys.argv[0])
        scriptPath = os.path.join(os.path.abspath(scriptPath) ,"")
        outDataPath = scriptPath + outDataPath

        configFileName = scriptPath + "config.txt"
        requiredFiles = [outDataPath + "goTermFeatureMap_11_3_12", 
                         outDataPath + "AllApicomplexan_uniqueStrand_11_28_12_forRealAll.goTermList",
                         outDataPath + "GESindex", 
                         outDataPath + "allApicomplexan_uniqueStrand_11_28_12.idTMtopology_signalpResolved",
                         configFileName]                                    
        requiredFileErrorFlag = requiredFileCheck(requiredFiles)
        errorMessage = "One or more essential file for ApicoAMP are missing!"
        
        if not requiredFileErrorFlag:
            configFile = open(configFileName,'r')
            SVM_binariesPath = configFile.readline().strip()
            SVM_binariesPath = os.path.join(scriptPath, os.path.join(SVM_binariesPath, "") )
            if not os.path.exists(SVM_binariesPath):
                requiredFileErrorFlag = True       
                errorMessage = "Given SVM path does not exist!"

        if not requiredFileErrorFlag:
            import glob
            if not glob.glob(SVM_binariesPath + "svm_classify*"):
                requiredFileErrorFlag = True
                errorMessage = "Given SVM path does not contain SVM binaries, read requirements section in README to find out how to get them."
                         
        if not requiredFileErrorFlag:  
            result = promptUser()
            if result:
                [fastaFileName, predictionOutputFileName, voteThreshold] = result
                
                errorInFasta = ""
                if fastaFileName and os.path.exists(fastaFileName):
                    fileInLines = open(fastaFileName, 'r').readlines()
                else:
                    errorInFasta = "Please provide a valid file to open"
                
                if errorInFasta == "": 
                    errorInFasta = isFasta(fileInLines)
                    
                if errorInFasta == "":
                    proteinSequences, idList = fetchIDandSequence(fileInLines)
                    x, outStrReturn = ApicoAMP_classify(scriptPath, outDataPath, SVM_binariesPath, proteinSequences, idList, voteThreshold)
                    predictionOutputFile = open(predictionOutputFileName,'w')
                    predictionOutputFile.write(outStrReturn)
                    predictionOutputFile.close()
                    print("ApicoAMP predictions are successfully generated, check out the output file!")
                else:
                    print(errorInFasta)
        else:
            print(errorMessage)
            print("Please refer to the README file for more information")  
            
                    
                
        
        
    
        
    

        
    
    
    
    
            


