'''
Created on May 7, 2011
Last updated: Jan 18, 2012

@author: Gokcen Cilingir
'''

import Tkinter, tkFileDialog
from Tkinter import W, E, S, N, GROOVE, END, HORIZONTAL, VERTICAL, LEFT
import tkFont
import re
import os
from ApicoAMP_CL import *

OUTPUT_ENTRY_WIDTH = 20

class Application(Tkinter.Frame):

    def __init__(self, scriptPath,  outDataPath, SVM_binariesPath, master=None):
        
        Tkinter.Frame.__init__(self, master)       
        self.grid(padx=10, pady=10)
        self.inputType = Tkinter.IntVar()        # Radio buttons on frmSequenceImport sets this variable 
        self.inputType.set(0)
        self.fastaFileName = Tkinter.StringVar() # Fasta file name entered by the user  
        self.outFileName = Tkinter.StringVar()   # Output file name entered by the user  
        self.outStr = Tkinter.StringVar()        # Entire prediction output is kept in this variable


        self.defaultVoteCount = Tkinter.StringVar()      
        
        self.waitLbl = None
        
        self.customTitleFont = tkFont.Font(family="Segoe UI", size=10, weight = 'bold')
        self.customButtonFont = tkFont.Font(family="Segoe UI", size=9, weight = 'bold')
        self.createWidgets()
        self.outDataPath = outDataPath
        self.scriptPath = scriptPath
        self.SVM_binariesPath = SVM_binariesPath
    def createWidgets(self):
        
        self.frmMain = Tkinter.Frame(self)  #Main frame
        self.frmMain.grid()
        
        #############################################################################
        # Setting up parameter frame widgets
        #############################################################################
        frmParameters = Tkinter.Frame(self.frmMain, borderwidth=3, relief=GROOVE)
        frmParameters.grid(sticky=S+E+N+W)
        Tkinter.Label(frmParameters, text = 'Set parameters', font = self.customTitleFont).grid(column=0, row=0, sticky=W)
              
        lbl = Tkinter.Label(frmParameters, text = 'Vote count: ')
        lbl.grid(column=0, row=1, sticky=W, pady=10)
        entry = Tkinter.Entry(frmParameters, textvariable = self.defaultVoteCount, width=5)
        entry.grid(column=2, row=1, sticky=E, padx=10, pady=10)
        
        entry.insert(0, "10")
        entry.configure(justify = 'right')
        
        lbl = Tkinter.Label(frmParameters, text = '(range: [6-10], 10 being most stringent for predicting a sequence as positive)', font = tkFont.Font(family="Segoe UI", size=9,  slant = 'italic'))
        lbl.grid(column=3, row=1, sticky=W, padx=10, pady=10)
        
        #############################################################################
        # Setting up import sequence frame widgets
        #############################################################################
        frmSequenceImport = Tkinter.Frame(self.frmMain, borderwidth=3, relief=GROOVE)
        frmSequenceImport.grid(sticky=S+E+N+W)
        
        Tkinter.Label(frmSequenceImport, text = 'Submit Protein Sequences', font = self.customTitleFont).grid(column=0, row=0, sticky=W)
        
        self.SequenceImportSourceRadioButtonList = []
        radioSequenceImportSource1 = Tkinter.Radiobutton(frmSequenceImport, text='Enter sequences here in fasta format',
                                  command=self.onChangeInput,
                                  variable=self.inputType, value=0)
        radioSequenceImportSource1.grid(sticky=W, pady=10)  
        self.SequenceImportSourceRadioButtonList.append(radioSequenceImportSource1)
        self.txtSequence = Tkinter.Text(frmSequenceImport, height=15, width=80)
        self.txtSequence.grid(row=2, sticky=E+W)
        self.entryScroll = Tkinter.Scrollbar (frmSequenceImport, orient=VERTICAL,
            command=self.__scrollHandler )
        self.entryScroll.grid(row=2, sticky=N+S+E)
        self.txtSequence["yscrollcommand"] = self.entryScroll.set
        
        radioSequenceImportSource2 = Tkinter.Radiobutton(frmSequenceImport, text='Read sequences from fasta file', 
                                  command=self.onChangeInput,
                                  variable=self.inputType, value=1)
        radioSequenceImportSource2.grid(sticky=W, pady=10)
        self.SequenceImportSourceRadioButtonList.append(radioSequenceImportSource2)
        
        self.frmFile = Tkinter.Frame(frmSequenceImport, padx=4)
        self.frmFile.grid(sticky=E + W)
        self.entryFileName = Tkinter.Entry(self.frmFile, textvariable=self.fastaFileName, width=80)
        self.entryFileName.grid(sticky=E+W)
        self.btnBrowse = Tkinter.Button(self.frmFile, text='Browse', command=self.onClickBrowse, font = self.customButtonFont)
        self.btnBrowse.grid(row=0, column=1, sticky=E, padx=18)
        
        self.btnClassify = Tkinter.Button(self.frmMain, text='Run ApicoAMP', command=self.onClickClassify, font = self.customButtonFont)
        self.btnClassify.grid(row=4, pady=8)
        self.btnClearForm = Tkinter.Button(self.frmMain, text='Clear text', command=self.onClickClearForm, font = self.customButtonFont)
        self.btnClearForm.grid(row=4, sticky=E)

        self.inputTypeWidgets = [[self.txtSequence, self.entryScroll, self.btnClearForm], [self.frmFile]] ## store for show/hide
        self.frmFile.grid_remove()
            
        #############################################################################
        # Creating error frame (initially hidden)
        #############################################################################
        
        self.errorFrame = Tkinter.Frame(self, padx=18, pady=8)
        self.errorDisplayFrame = Tkinter.Frame(self.errorFrame, borderwidth=2, relief=GROOVE)
        self.errorDisplayFrame.grid()
        
        self.errorLabel = Tkinter.Label(self.errorDisplayFrame, text="")
        self.errorLabel.grid(column=0, row=0)
        
        Tkinter.Button(self.errorDisplayFrame, text='< Back', command=self.onClickErrorBack, font = self.customButtonFont).grid(pady=8)
               
    def onClickClearForm(self):
        if (self.inputType.get()  == 0):
            self.txtSequence.delete('1.0', END)
        else:
            self.fastaFileName.set("")
   
                     
    def waitPromptDestroy(self):
        if self.waitLbl:
            self.waitLbl.destroy() 
            self.frmMain.update()
    
    def waitPromptCreate(self, text):
        self.waitLbl = Tkinter.Label(self.frmMain, text = text, fg = 'red')
        self.waitLbl.grid()
        self.frmMain.update()

    def waitPromptUpdate(self, text):
        self.waitLbl.configure(text=text)
        self.waitLbl.grid() #?
        self.frmMain.update()
             
    def disableEverything(self):
        self.btnClassify.config(state=Tkinter.DISABLED)
        self.btnClearForm.config(state=Tkinter.DISABLED)

        self.btnBrowse.config(state=Tkinter.DISABLED)
        for b in self.SequenceImportSourceRadioButtonList:
            b.config(state=Tkinter.DISABLED)

            
    def enableEverything(self):
        self.btnClassify.config(state=Tkinter.NORMAL)
        self.btnClearForm.config(state=Tkinter.NORMAL)
        #self.btnImportFiles.config(state=Tkinter.NORMAL)
        self.btnBrowse.config(state=Tkinter.NORMAL)
        for b in self.SequenceImportSourceRadioButtonList:
            b.config(state=Tkinter.NORMAL)
         
    def onClickClassify(self):
    
    
        self.disableEverything()  

        # Get protein sequences entered (either to the text box or read from a file)
        errorInFasta = ""
        if (self.inputType.get()  == 0):
            txt = self.txtSequence.get('1.0', END)          
            fileInLines = txt.split("\n")         
        else:
            filename = self.fastaFileName.get()
            if filename and os.path.exists(filename):
                fileInLines = open(filename, 'r').readlines()
            else:
                errorInFasta = "File does not exist, please provide a valid file to open"
        
        if int(self.defaultVoteCount.get()) < 6 or int(self.defaultVoteCount.get()) > 10:
            errorInFasta = "Parameter set is not in the expected range [6-10], please set it appropriately"
        if errorInFasta == "": 
            errorInFasta = isFasta(fileInLines) 
                   
        if errorInFasta != "":
            self.frmMain.grid_forget()  # hide entry frame
            self.errorLabel.configure(text=errorInFasta)
            self.errorFrame.grid(sticky=N+S+E+W) # show error frame
        else:
            self.waitPromptCreate('Please wait until execution is completed.')
            proteinSequences, idList = fetchIDandSequence(fileInLines)
            
            idPredictionBundle, x = ApicoAMP_classify(self.scriptPath, self.outDataPath, self.SVM_binariesPath, proteinSequences, idList, int(self.defaultVoteCount.get()))
            formatStr = "{0:" + str(OUTPUT_ENTRY_WIDTH) + "}"
            
            self.outStr = formatStr.format("ID") 
            self.outStr += formatStr.format("Prediction") 
            self.outStr += formatStr.format("TMC_voteCount") 
            self.outStr += formatStr.format("GOC_result") 
            
            self.outStr += "\n\n"
            for i in range(len(idList)):
                id = idList[i]
                pred = PREDICTION_CODES[idPredictionBundle[id].TMstatus]
                TMC_vote = idPredictionBundle[id].TM_classifierOut
                GOC_result = PREDICTION_CODES[idPredictionBundle[id].GO_classifierOut]
                GOterms = "/".join(idPredictionBundle[id].GOterms)
                self.outStr += formatStr.format(str(id))
                self.outStr += formatStr.format(str(pred))
                self.outStr += formatStr.format(str(TMC_vote))
                self.outStr += formatStr.format(str(GOC_result))
                #self.outStr += formatStr.format(GOterms)
                
                if(i != len(idList)-1):
                    self.outStr += '\n' 
                        
                        
            self.frmPredictions = Tkinter.Frame(self.frmMain, borderwidth=2, relief=GROOVE)
            self.frmPredictions.grid()
                
            self.txtSequenceP = Tkinter.Text(self.frmPredictions, height=15, width=80)
            self.txtSequenceP.grid(row=1, sticky=E+W)
    
            self.entryScrollP = Tkinter.Scrollbar (self.frmPredictions, orient=VERTICAL,
                command=self.__scrollHandlerP )
            self.entryScrollP.grid(row=1, sticky=N+S+E)
            self.txtSequenceP["yscrollcommand"] = self.entryScrollP.set
            
            writeToFile = Tkinter.Button(self.frmPredictions, text='Save predictions', command=self.onClickWriteToFile, font = self.customButtonFont)
            writeToFile.grid(row = 2,column = 0, padx = 60, pady=8, sticky=W)
            
            self.txtSequenceP.insert(END, self.outStr)
            self.frmMain.grid()
            
            btnBack = Tkinter.Button(self.frmPredictions, text='Enter new data', command=self.onClickBack, font = self.customButtonFont)
            btnBack.grid(row=2, column = 0, padx = 60, pady=8, sticky=E)

        self.waitPromptDestroy() 
         

    def onClickBack(self):
        self.frmPredictions.grid_forget()
        self.enableEverything()
        self.fastaFileName.set("") 
        
    def onClickErrorBack(self):
        self.errorFrame.grid_remove()
        self.txtSequence.delete('1.0', END)  
        
        self.frmMain.grid()
        self.enableEverything() 
                  
    def onClickBrowse(self):
        self.fastaFileName.set(tkFileDialog.askopenfilename())
        
    def onClickWriteToFile(self):
        fileName = tkFileDialog.asksaveasfilename()
        if(fileName):
            outfile = open(fileName,"w")
            outfile.write(self.outStr)
            outfile.close()    
     
    #Disabled due to SIGNALP integration        
    '''def onChangeSpImport(self):
        for i in range(len(self.spImportWidgets)):
            if i != self.spVar.get():
                for widget in self.spImportWidgets[i]:
                    widget.grid_remove()
            else:
                for widget in self.spImportWidgets[i]:
                    widget.grid()
    '''                  
    def onChangeInput(self):
        idx = self.inputType.get()
        for widget in self.inputTypeWidgets[1 - idx]:
            widget.grid_remove()
        for widget in self.inputTypeWidgets[idx]:
            widget.grid()
    
    def __scrollHandler(self, *L):
        op, howMany = L[0], L[1]
        if op == "scroll":
            units  =  L[2]
            self.txtSequence.yview_scroll ( howMany, units )
        elif op == "moveto":
            self.txtSequence.yview_moveto ( howMany )
    
    def __scrollHandlerP(self, *L):
        op, howMany = L[0], L[1]
        if op == "scroll":
            units  =  L[2]
            self.txtSequenceP.yview_scroll ( howMany, units )
        elif op == "moveto":
            self.txtSequenceP.yview_moveto ( howMany )

class SplashScreen(Tkinter.Toplevel):
    def __init__(self, master, image=None, timeout=3000):
        """
        create a splash screen from a specified image file
        keep splash screen up for timeout milliseconds
        """
        Tkinter.Toplevel.__init__(self, master, relief='raised', borderwidth=5)
        self.main = master

        # don't show main window
        self.main.withdraw()
        self.overrideredirect(1)

        # use Tkinter's PhotoImage for .gif files
        self.image = Tkinter.PhotoImage(file=image)
        self.after_idle(self.centerOnScreen)

        self.update()
        self.after(timeout, self.destroySplash)

    def centerOnScreen(self):
        self.update_idletasks()
        self.width, self.height = self.image.width(), self.image.height()

        xmax = self.winfo_screenwidth()
        ymax = self.winfo_screenheight()

        x0 = self.x0 = xmax/2 - self.width/2
        y0 = self.y0 = ymax/2 - self.height/2
        self.geometry("%dx%d+%d+%d" % (self.width, self.height, x0, y0))
        self.createSplash()

    def createSplash(self):
        # show the splash image
        self.canvas = Tkinter.Canvas(self, height=self.height, width=self.width)
        self.canvas.create_image(0,0, anchor='nw', image=self.image)
        self.canvas.pack()

    def destroySplash(self):
        # bring back main window and destroy splash screen
        self.main.update()
        self.main.deiconify()
        self.withdraw()

import sys

if __name__ == "__main__":
    
    req_version = (2,7)
    cur_version = sys.version_info
    if cur_version.major != req_version[0] or cur_version.minor != req_version[1] :
        print "Python 2.7 is required for ApicoAMP to work. You are using Python " + str(cur_version.major) +  "." + str(cur_version.minor) + ". " + \
        "Please make sure you're using the correct Python version."
    else:
        
        ################################################################################
        # Reading config file into PredictionInputUnit data structures
        # Checking if required files (given in config file) and splash screen figure exist
        # Reading rule files into Rule data structures
        # Starting SplashScreen and the main Application
        ################################################################################
    
        scriptPath, scriptname = os.path.split(sys.argv[0])
        scriptPath = os.path.join(os.path.abspath(scriptPath) ,"")
        outDataPath = scriptPath + outDataPath

        
        configFileName = scriptPath + "config.txt"
        requiredFiles = [outDataPath + "goTermFeatureMap_11_3_12", 
                         outDataPath + "AllApicomplexan_uniqueStrand_11_28_12_forRealAll.goTermList",
                         outDataPath + "GESindex", 
                         outDataPath + "allApicomplexan_uniqueStrand_11_28_12.idTMtopology_signalpResolved",
                         configFileName]                                    
        requiredFileErrorFlag = requiredFileCheck(requiredFiles)
        errorMessage = "One or more essential file for ApicoAMP are missing!"
        
        splashImageFile = scriptPath + "GUI_splash_ApicoAMP.gif"      
        if not os.path.exists(splashImageFile):
            requiredFileErrorFlag = True
            errorMessage = "GUI_splash_ApicoAMP.gif is missing!"
        
        if not requiredFileErrorFlag:
            configFile = open(configFileName,'r')
            SVM_binariesPath = configFile.readline().strip()
            SVM_binariesPath = os.path.join(scriptPath, os.path.join(SVM_binariesPath, "") )
            if not os.path.exists(SVM_binariesPath):
                requiredFileErrorFlag = True
                errorMessage = "Given SVM path does not exist!"
                
            import glob
            if not glob.glob(SVM_binariesPath + "svm_classify*"):
                requiredFileErrorFlag = True
                errorMessage = "Given SVM path does not contain SVM binaries, read requirements section in README to find out how to get them."
            
        if requiredFileErrorFlag:
            root = Tkinter.Tk()
            root.resizable(width=False, height=False)
    
            lbl = Tkinter.Label(root, text = errorMessage)
            lbl.grid(row=0, column=0, padx=5, pady=5)
            lbl2 = Tkinter.Label(root, text="Please refer to the README file for more information")
            lbl2.grid(row=1, column=0, padx=5, pady=5)
            root.mainloop()
        
        else:
            root = Tkinter.Tk()
            root.resizable(width=False, height=False)
            splashTimeout = 4000
            s = SplashScreen(root, timeout=splashTimeout, image=splashImageFile)
            app = Application(scriptPath, outDataPath, SVM_binariesPath, root)
            app.master.title("ApicoAMP Classifier")
            app.mainloop()
            root.mainloop()
    
    
    
    
    
